@@ -1,19 +1,16 @@
 use strict; use warnings;
 package YAMLScript::Main;
 
-# XXX Local dev lib:
-use lib "$ENV{HOME}/src/lingy/perl/lib";
-
 use Lingy::Common;
 use YAMLScript::Reader;
 
 BEGIN {
     package Lingy::Common;
     no warnings 'redefine';
-    sub RT() { 'YAMLScript::Lang::RT' }
+    sub RT() { 'YAMLScript::RT' }
 }
 
-use YAMLScript::Lang::RT;
+use YAMLScript::RT;
 use Lingy::Main;
 
 use base 'Lingy::Main';
