@@ -129,6 +129,7 @@ sub parse_yaml_pp {
 #------------------------------------------------------------------------------
 
 my $bp = $RE{balanced}{-parens=>'()'};
+my $bs = $RE{balanced}{-parens=>'[]'};
 
 my $E_GROUP = 'event'->new("=xxx\t-1\t-1\t-1\t-1\t-1\t-1\t-\t-\t-\t-");
 my $E_PLAIN = 'event'->new("=xxx\t-1\t-1\t-1\t-1\t-1\t-1\t-\t-\t:\t-");
@@ -139,15 +140,15 @@ sub SEQ { 'seq'->new($E_GROUP, @_) }
 sub VAL { 'val'->new($E_PLAIN, @_) }
 sub STR { 'val'->new($E_QUOTE, @_) }
 
-sub B { boolean($_[0]) }
-sub K { keyword(@_) }
-sub L { list([@_]) }
-sub N { number(@_) }
-sub S { symbol($_[0]) }
-sub T { string(@_) }
-sub V { vector([@_]) }
+sub B { BOOLEAN->new($_[0]) }
+sub K { KEYWORD->new(@_) }
+sub L { LIST->new([@_]) }
+sub N { NUMBER->new($_[0]) }
+sub S { SYMBOL->new($_[0]) }
+sub T { STRING->new($_[0]) }
+sub V { VECTOR->new([@_]) }
 
-sub DEF { S 'def!' }
+sub DEF { S 'def' }
 sub DO { S 'do' }
 sub FN { S 'fn*' }
 sub IF { S 'if' }
@@ -302,9 +303,15 @@ sub construct_defn_multi($s, $p) {
     return L($def, $name, @defs);
 }
 
+sub construct_fn($s, $p) {
+    my ($k, $v) = @$p;
+    my ($def, $name, $args, $body) = $s->_defn_parse($k, $v);
+    return L(FN, V(@$args), @$body);
+}
+
 sub _defn_parse($s, $k, $v) {
     my $macro = 0;
-    text($k) =~ /^($sym?)\((.*)\)$/ or die;
+    text($k) =~ /^($sym?)?\((.*)\)$/ or die;
     my $name = S($1);
     my $sig = $2;
     if ($sig =~ s/^\(//) {
@@ -314,7 +321,7 @@ sub _defn_parse($s, $k, $v) {
     }
     my ($args, $dargs) = get_sig($sig);
     my $defn = L( DEF, $name, L( FN, L, nil ) );
-    my $seq = is_seq($v) ? $v: SEQ($v);
+    my $seq = is_seq($v) ? $v : SEQ($v);
     my $first = first_elem($seq);
     my $body = [
         (@$dargs or is_def($first) or is_map($first))
@@ -373,18 +380,6 @@ sub construct_keyword($s, $n) {
     K("$n");
 }
 
-sub construct_lambda($s, $p) {
-    my ($k, $v) = @$p;
-    text($k) =~ /^\\\((.*)\)$/ or die;
-    my $sig = V(map symbol($_), split /\s+/, $1);
-    my $seq = is_seq($v) ? $v : SEQ($v);
-    my $first = first_elem($seq);
-    my @body = (is_def($first) or is_map($first))
-        ? ($s->construct_let($seq, [], []))
-        : map $s->construct($_), @{$seq->elem};
-    return L(FN, $sig, @body);
-}
-
 sub construct_let($s, $n, $a, $d) {
     my @elems = elems($n);
     if (is_map($elems[0]) and @{$elems[0]->{pair}} > 1) {
@@ -442,8 +437,16 @@ sub construct_let1($s, $n) {
     );
 }
 
-sub construct_ysexpr($s, $n) {
-    read_ysexpr($n);
+sub construct_loop($s, $p) {
+    my ($k, $v) = @$p;
+    "$k" =~ /^loop +($bs)/ or die;
+    my $bindings = read_ysexpr($1);
+    my @elems = is_seq($v) ? elems($v) : $v;
+    L(
+        S('loop'),
+        $bindings,
+        map $s->construct($_), @elems,
+    );
 }
 
 sub construct_module($s, $n) {
@@ -492,6 +495,10 @@ sub construct_val($s, $n) {
     T("$n");
 }
 
+sub construct_ysexpr($s, $n) {
+    read_ysexpr($n);
+}
+
 sub is_main($n) {
     ref($n) eq LIST and
     @$n >= 2 and
@@ -748,8 +755,9 @@ sub tag_node($n) { o;
             tag_defn($p) or
             tag_def($p) or
             tag_if($p) or
-            tag_lambda($p) or
+            tag_fn($p) or
             tag_let($p) or
+            tag_loop($p) or
             tag_try($p) or
             tag_call($p) or
             XXX $p, "Unable to implicitly tag this map pair.";
@@ -815,7 +823,7 @@ sub tag_def($p) {
 
 sub tag_defn($p) {
     my ($k, $v) = @$p;
-    return unless $k =~ /^$sym\((.*)\)$/;
+    return unless $k =~ /^$sym$bp$/;
     $k->{ytag} = 'defn';
     tag_node($v);
 }
@@ -845,14 +853,24 @@ sub tag_istr($n) {
     $n->{ytag} = 'istr' if $n =~ /(\$$sym|\$\()/;
 }
 
-sub tag_lambda($n) {
-    $n->{ytag} = 'lambda' if $n =~ /^\\\((.*)\)$/;
+sub tag_fn($p) {
+    my ($k, $v) = @$p;
+    return unless $k =~ /^$bp$/;
+    $k->{ytag} = 'fn';
+    tag_node($v);
 }
 
 sub tag_let($n) {
     $n->{ytag} = 'let1' if $n =~ /^let$/;
 }
 
+sub tag_loop($p) { o;
+    my ($k, $v) = @$p;
+    return unless $k =~ /^loop +\S/;
+    $k->{ytag} = 'loop';
+    tag_node($v);
+}
+
 sub tag_scalar($n) {
     local $_ = $n;
     $n->{ytag} =
